// ImageA46.pov for POV-Ray Megapatch 0.7: Evening at the River  (c) 2002 Ch. Gerber

#version unofficial MegaPov 0.7;

#include "colors.inc"
#include "mytextures.inc"

#declare HQ        = 1;
#declare Clouds_on = 1;
#declare Forest_on = 1;

#if (HQ=1) $ mIntervals = 2; $ mSamples = 8; #else $ mIntervals = 1; $ mSamples = 4; #end
 
global_settings {assumed_gamma 2.6 max_trace_level 15}


// --- Camera, Sun and Sky ---

#declare Cam_pos    = <0,25,-25>;
#declare Cam_look_at = <0,50,250>;
#declare Cam_dir = z;
#declare Sun_dist = 1000000;
#declare Sun_size = 50000;
#declare Flare_size = 1.0;
#declare Sun_az = -18.5;
#declare Sun_alt = 12.5;
#declare Sun_color = rgb <0.996,0.996,0.781>; 
#declare Sun_brightness = 1.3;
#declare Sky_color = rgb <0.591,0.577,0.647>;
#declare Flare_color = (Sun_color + White)/2;
#declare Ground_rad =25000;

camera {
  location  Cam_pos
  direction Cam_dir
  right 4/3*x
  look_at Cam_look_at
}

light_source { 
  z*Sun_dist  Sun_color*Sun_brightness
  rotate -x*Sun_alt  rotate y*Sun_az 
} 

light_source {
  <-100,500,-1000>
  color rgb 0.1
  media_attenuation off 
  media_interaction off
  shadowless
}

light_source {
  <-1000,2000,-1000>
  color rgb 0.2
  media_attenuation off
  media_interaction off
}

sky_sphere {
  pigment {
    gradient y 
    color_map {[0.5 Sun_color*0.9][0.7 Sky_color]} 
    scale 2 
    translate -1*y
    warp {black_hole vaxis_rotate(vaxis_rotate(z,-x,Sun_alt),y,Sun_az), 1 inverse strength 1.5}
  }
  pigment {
    gradient y
    color_map  {[0.0 rgbt <1.0,0.5,0.2,0.5>][0.3 rgbt <1.0,0.5,0.2,1.0>]}
  }  
}

disc {
  0, z, Sun_size no_shadow hollow
  pigment {spherical color_map{[0 rgbt 1][0.2 Sun_color*Sun_brightness*0.9]} scale Sun_size/2} 
  finish {ambient 1} 
  translate Sun_dist*z rotate -Sun_alt*x rotate Sun_az*y 
}

disc { 
  0, Cam_dir, 1 no_shadow hollow
  pigment {spherical color_map {[0 rgbf 1][1 Flare_color*Sun_brightness*0.9 filter 0.3]} cubic_wave}
  finish {ambient 1}
  scale 0.001*Flare_size
  translate vnormalize(Cam_dir)*0.01 rotate -Sun_alt*x rotate Sun_az*y translate Cam_pos 
}

fog {fog_type 2 color Sun_color distance 500 fog_alt 35 fog_offset 0 turbulence 1}     

#if (Clouds_on = 1)
difference {
  sphere {0, Ground_rad + 600}
  sphere {0, Ground_rad + 350}
  pigment {rgbf 1}
  interior {
    media {
      scattering {2, <0.01,0.012,0.01> extinction 4.5}
      method 3
      intervals mIntervals
      samples mSamples, mSamples 
      density {
        wrinkles
        ramp_wave
        color_map {
          [0   rgb 0]
          [0.5 rgb 0]
          [0.7 rgb 0.4]
          [1   rgb 0.7]
        }
        scale <67,33,60>
        rotate y*25
        translate 85*y
      }
      density {
        marble
        triangle_wave
        turbulence 1.5
        color_map {
          [0    rgb 0]
          [0.4  rgb 0]
          [0.85 rgb 0.35]
          [1    rgb 0.7]
        }
        scale <1000,500,750>
        rotate -25*y
        translate 85*y
      }
    }
  } 
  translate -Ground_rad*y
  scale <1,0.5,1>
  hollow
}
#end    


//--- Landscape and Forest ---

#declare Slope = texture  {
  slope y
  texture_map {
    [0.0  RockTex]
    [0.89 RockTex]
    [0.99 GroundTex]
    [1.0  GroundTex]
  }
}

#declare River = height_field { 
  tga "river.tga" smooth translate -0.5*x
  scale <1000,60,1000>
  translate -130*x
  texture  {
    gradient y
    texture_map {
      [0.0  StoneTex]
      [0.03 StoneTex]
      [0.04 Slope]
      [1.0  Slope]
    }
    scale 60
    warp {reset_children}
  }
}

plane {y,2 texture {WaterTex}}

#declare Hills = height_field { 
  tga "hills.tga" smooth translate -0.5*x
  scale <2000,95,2000>
  texture {
    slope y
    texture_map {
      [0.0  RockTex]
      [0.85 RockTex]
      [0.95 GroundTex]
      [1.0  GroundTex]
    }
  }
}

object {River}
object {Hills translate < -400,0,1000>}

#if (Forest_on = 1)

#include "oakmesh.inc"

#declare Oak01 = union {
  object {Oak_BranchesIteration1 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration2 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration3 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration4 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration5 texture {Oak_TrunkTexPlain}}
  object {Oak_BranchesIteration6 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_BranchesIteration7 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_Leaves uv_mapping  texture {Oak_LeafTexGreen} double_illuminate}
  scale 5
}

#declare Oak02 = union {
  object {Oak_BranchesIteration1 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration2 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration3 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration4 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration5 texture {Oak_TrunkTexPlain}}
  object {Oak_BranchesIteration6 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_BranchesIteration7 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_Leaves uv_mapping  texture {Oak_LeafTexBrown} double_illuminate} 
  scale 5
}

#declare Oak03 = union {
  object {Oak_BranchesIteration1 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration2 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration3 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration4 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration5 texture {Oak_TrunkTexPlain}}
  object {Oak_BranchesIteration6 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_BranchesIteration7 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_Leaves uv_mapping  texture {Oak_LeafTexRed} double_illuminate}
  scale 5
}

#declare Oak04 = union {
  object {Oak_BranchesIteration1 uv_mapping texture {Oak_TrunkTexPlain}}
  object {Oak_Leaves uv_mapping  texture {Oak_LeafTexGreen} double_illuminate}
  scale 5
}

#declare Bush = object {
  Oak_Leaves uv_mapping  texture {Oak_LeafTexGreen} double_illuminate
  scale <5,3,5> translate -4*y
}  

#macro SlopeTest(Norm,MaxSlope)
  #declare STest = false;
  #if (vlength(Norm)!=0)
    #declare STest = true;
    #local Test = acos(vdot(Norm,y)/(vlength(Norm)*vlength(y)))*(180/pi);
    #if (Test > MaxSlope) #if (Test < 180-MaxSlope) #declare STest = false; #end #end
  #end  
#end
   
#declare Norm = <0,0,0>;
#declare Xpos = -630;
#declare Zpos = 0;
#declare MaxSlope = 25;
#declare MaxHeight = 60;
#declare MinHeight = 2.5;
#declare VegeSeed = seed(2002);
#declare Pig = pigment {spotted color_map {[0 rgb 1][1 rgb 0]} scale 10 translate <-50,0,250>}
#while (Xpos<370)
  #while (Zpos<1000)
    #declare SPos = <Xpos,100,Zpos>;
    #declare PP = eval_pigment(Pig,<Xpos,0,Zpos>);
    #declare XX = PP.x;
    #declare Inter = trace (River,SPos,<0,-1,0>,Norm);
    SlopeTest(Norm,MaxSlope)
    #if (STest=true)
      #if (Inter.y<MaxHeight & Inter.y>MinHeight) 
        #switch (XX)
          #range (0.7,1.0) 
            object {
              Oak03
              scale 0.7+0.6*rand(VegeSeed)
              rotate 360*rand(VegeSeed)*y 
              translate <Inter.x+rand(VegeSeed)*5,Inter.y,Inter.z+rand(VegeSeed)*5> 
            }  
          #break
          #range (0.5,0.7)
            object {
              Oak02
              scale 0.7+0.6*rand(VegeSeed)
              rotate 360*rand(VegeSeed)*y 
              translate <Inter.x+rand(VegeSeed)*5,Inter.y,Inter.z+rand(VegeSeed)*5>
            }
          #break
          #else
            object {
              Oak01
              scale 0.7+0.6*rand(VegeSeed)
              rotate 360*rand(VegeSeed)*y 
              translate <Inter.x+rand(VegeSeed)*5,Inter.y,Inter.z+rand(VegeSeed)*5>
            }
        #end  
      #end 
    #end  
    #declare Zpos = Zpos + 10;
  #end           
  #declare Zpos = 0;
  #declare Xpos = Xpos + 10; 
#end

#declare Norm = <0,0,0>;
#declare Xpos = -200;
#declare Zpos = 0;
#declare MaxSlope = 60;
#declare MaxHeight = 5;
#declare MinHeight = 2;
#declare VegeSeed = seed(1);
#while (Xpos<200)
  #while (Zpos<400)
    #declare SPos = <Xpos,100,Zpos>;
    #declare Inter = trace (River,SPos,<0,-1,0>,Norm);
    SlopeTest(Norm,MaxSlope)
    #if (STest=true)
      #if (Inter.y<MaxHeight & Inter.y>MinHeight) 
        object {
          Bush
          scale 0.7+0.6*rand(VegeSeed)
          rotate 360*rand(VegeSeed)*y 
          translate <Inter.x+rand(VegeSeed)*5,Inter.y,Inter.z+rand(VegeSeed)*5> 
        }  
      #end
    #end    
    #declare Zpos = Zpos + 5;
  #end           
  #declare Zpos = 0;
  #declare Xpos = Xpos + 5; 
#end

#declare Norm = <0,0,0>;
#declare Xpos = -1000;
#declare Zpos = 0;
#declare MaxSlope = 25;
#declare MaxHeight = 95;
#declare MinHeight = 10;
#declare VegeSeed = seed(69);
#while (Xpos<1000)
  #while (Zpos<2000)
    #declare SPos = <Xpos,200,Zpos>;
    #declare Inter = trace (Hills,SPos,<0,-1,0>,Norm);
    SlopeTest(Norm,MaxSlope)
    #if (STest=true)
      #if (Inter.y<MaxHeight & Inter.y>MinHeight) 
        object {
          Oak04
          scale 0.7+0.6*rand(VegeSeed)
          rotate 360*rand(VegeSeed)*y 
          translate <Inter.x+rand(VegeSeed)*5,Inter.y,Inter.z+rand(VegeSeed)*5>
          translate <-400,0,1000> 
        }  
      #end
    #end    
    #declare Zpos = Zpos + 10;
  #end           
  #declare Zpos = 0;
  #declare Xpos = Xpos + 10; 
#end

#end //Forest_on

